==> apgl-0.6.1.tar.gz: apgl-0.6.1/apgl/predictors/test/LibSVMTest.py <==
'''
Created on 24 Jul 2009

@author: charanpal
'''
import unittest
import logging
import numpy
from apgl.predictors.LibSVM import LibSVM
from apgl.data import * 
from apgl.util import *

class LibSVMTest(unittest.TestCase):
    def setUp(self):
        try:
            import svm
        except ImportError as error:
            print(error)
            return 

        numExamples = 100
        numFeatures = 10
        eg = ExamplesGenerator()

        self.X, self.y = eg.generateBinaryExamples(numExamples, numFeatures)
        self.svm = LibSVM()

    def tearDown(self):
        pass

    def testLearnModel(self):
        try:
            import svm
        except ImportError as error:
            return

        self.svm.learnModel(self.X, self.y)
        predY, decisions = self.svm.classify(self.X)
        y = self.y

        e = Evaluator.binaryError(y, predY)
        logging.info("Error rate : "  + str(e))

        #Test for wrong labels
        numExamples = 6
        X = numpy.array([[-3], [-2], [-1], [1], [2] ,[3]], numpy.float)
        y = numpy.array([[-1], [-1], [-1], [1], [1] ,[5]])


        self.assertRaises(ValueError, self.svm.learnModel, X, y)

    def testSetErrorCost(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 1000
        numFeatures = 100
        eg = ExamplesGenerator()
        X, y = eg.generateBinaryExamples(numExamples, numFeatures)
        svm = LibSVM()

        C = 0.1
        kernel = "linear"
        kernelParam = 0
        svm.setKernel(kernel, kernelParam)
        svm.setC(C)

        svm.setErrorCost(0.1)
        svm.learnModel(X, y)
        predY, decisions = svm.classify(X)
        e1 = Evaluator.binaryErrorP(y, predY)

        svm.setErrorCost(0.9)
        svm.learnModel(X, y)
        predY, decisions = svm.classify(X)
        e2 = Evaluator.binaryErrorP(y, predY)

        self.assertTrue(e1 > e2)

    def testClassify(self):
        try:
            import svm
        except ImportError as error:
            return

        self.svm.learnModel(self.X, self.y)
        predY, decisions = self.svm.classify(self.X)
        y = self.y

        e = Evaluator.binaryError(y, predY)

        #Now, permute examples
        perm = numpy.random.permutation(self.X.shape[0])
        predY, decisions = self.svm.classify(self.X[perm, :])
        y = y[perm]

        e2 = Evaluator.binaryError(y, predY)

        self.assertEquals(e, e2)

    def testEvaluateCv(self):
        try:
            import svm
        except ImportError as error:
            return

        folds = 10
        (means, vars) = self.svm.evaluateCv(self.X, self.y, folds)

        self.assertTrue((means <= 1).all())
        self.assertTrue((means>= 0).all())
        self.assertTrue((vars <= 1).all())
        self.assertTrue((vars>= 0).all())

    def testCvModelSelection(self):
        try:
            import svm
        except ImportError as error:
            return

        kernel = "linear"
        kernelParams = [0.1, 0.2, 0.4]
        Cs = [1.0, 0.5, 0.25, 0.125]
        errorCosts = [0.8, 0.9]
        folds = 5

        results = self.svm.cvModelSelection(self.X, self.y,Cs, kernelParams, kernel, folds, errorCosts)

        self.assertTrue(results["C"] in Cs)
        self.assertTrue(results["kp"] in kernelParams)
        self.assertTrue(results["errorCost"] in errorCosts)
        self.assertTrue(results["error"] >= 0 and results["error"] <= 1)
        

        #Test gaussian kernel
        kernel= "gaussian"
        results = self.svm.cvModelSelection(self.X, self.y ,Cs, kernelParams, kernel, folds, errorCosts)

        self.assertTrue(results["C"] in Cs)
        self.assertTrue(results["kp"] in kernelParams)
        self.assertTrue(results["errorCost"] in errorCosts)
        self.assertTrue(results["error"] >= 0 and results["error"] <= 1)


        #Test polynomial kernel
        kernel= "polynomial"
        kernelParams = [1, 2, 3]
        results = self.svm.cvModelSelection(self.X, self.y,Cs, kernelParams, kernel, folds, errorCosts)

        self.assertTrue(results["C"] in Cs)
        self.assertTrue(results["kp"] in kernelParams)
        self.assertTrue(results["errorCost"] in errorCosts)
        self.assertTrue(results["error"] >= 0 and results["error"] <= 1)

        #Try a different error function
        results = self.svm.cvModelSelection(self.X, self.y,Cs, kernelParams, kernel, folds, errorCosts, errorMethod=Evaluator.balancedError)

    def testGetModel(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 50
        numFeatures = 3
        eg = ExamplesGenerator()

        X, y = eg.generateBinaryExamples(numExamples, numFeatures)
        svm = LibSVM()
        svm.learnModel(X, y)

        weights, b  = svm.getWeights()

        print(weights)
        print(b)

    def testGetWeights(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 6
        X = numpy.array([[-3], [-2], [-1], [1], [2] ,[3]], numpy.float64)
        y = numpy.array([[-1], [-1], [-1], [1], [1] ,[1]])

        svm = LibSVM()
        svm.learnModel(X, y.ravel())

        weights, b  = svm.getWeights()

        self.assertTrue(weights[0] > 0)

        #Let's see if we can compute the decision values 
        y, decisions = svm.classify(X)

        decisions2 = numpy.zeros(numExamples)

        for i in range(numExamples):
            decisions2 = numpy.dot(X, weights) - b

        self.assertTrue((decisions == decisions2).all())
        predY = numpy.sign(decisions2)

        self.assertTrue((y.ravel() == predY).all())

    def testSetEpsilon(self):
        try:
            import svm
        except ImportError as error:
            return

        print("Learning model with epsilon as default")
        self.svm.learnModel(self.examplesList)

        print("Learning model with epsilon as 0.1")
        self.svm.setTermination(0.1)
        self.svm.learnModel(self.examplesList)
        print("Done")

    def testSetSvmType(self):
        try:
            import svm
        except ImportError as error:
            return

       
        numExamples = 100
        numFeatures = 10
        X = numpy.random.randn(numExamples, numFeatures)
        c = numpy.random.randn(numFeatures)

        y = numpy.dot(X, numpy.array([c]).T).ravel()
        y2 = numpy.array(y > 0, numpy.int32)*2 -1 
        
        svm = LibSVM()

        svm.setSvmType("Epsilon_SVR")
        svm.learnModel(self.X, self.y)
        yp = svm.predict(X)
        print("Error on real labels ")
        print(Evaluator.rootMeanSqError(y, yp))

        #The values of y look correct so it looks like its working 
        #print(y[0:5])
        #print(yp[0:5])

        svm.setSvmType("C_SVC")
        svm.learnModel(X, y2)
        yp2 = svm.predict(X)

        print("Error on predicted labels ")
        print(Evaluator.binaryError(y2, yp2))
        print(y2[0:5])
        print(yp2[0:5])

    def testCvModelSelection2(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 100
        numFeatures = 10
        X = numpy.random.randn(numExamples, numFeatures)
        c = numpy.random.randn(numFeatures)

        y = numpy.dot(X, numpy.array([c]).T).ravel()

        examplesList = ExamplesList(numExamples)
        examplesList.addDataField("X", X)
        examplesList.addDataField("y", y)
        examplesList.setDefaultExamplesName("X")
        examplesList.setLabelsName("y")

        kernel = "linear"
        kernelParams = [0.1, 0.2, 0.4]
        Cs = [1.0, 0.5, 0.25, 0.125]
        errorCosts = [1.0]
        epsilons = [0.05, 0.1, 0.2, 0.5]
        folds = 5

        svm = LibSVM()
        svm.setSvmType("Epsilon_SVR")
        best = svm.cvModelSelection(self.X, self.y,Cs, kernelParams, kernel, folds, errorCosts, epsilons=epsilons, errorMethod=Evaluator.rootMeanSqError)

        print(best)

    def testSaveParams(self):
        try:
            import svm
        except ImportError as error:
            return

        svm = LibSVM()
        svm.setC(10.5)
        svm.setEpsilon(12.1)
        svm.setErrorCost(1.8)
        svm.setSvmType("Epsilon_SVR")
        svm.setTermination(0.12)
        svm.setKernel("gaussian", 0.43)

        outputDir = PathDefaults.getOutputDir()
        fileName = outputDir + "test/testSvmParams"
        svm.saveParams(fileName)

        svm2 = LibSVM()
        svm2.loadParams(fileName)

        self.assertEquals(svm.getC(), 10.5)
        self.assertEquals(svm.getEpsilon(), 12.1)
        self.assertEqual(svm.getErrorCost(), 1.8)
        self.assertEqual(svm.getSvmType(), "Epsilon_SVR")
        self.assertEqual(svm.getTermination(), 0.12)
        self.assertEqual(svm.getKernel(), "gaussian")
        self.assertEqual(svm.getKernelParams(), 0.43)

    def testStr(self):
        try:
            import svm
        except ImportError as error:
            return

        svm = LibSVM()

        print(svm)

    def testSetEpsilon(self):
        """
        Test out the parameter for the regressive SVM, vary epsilon and look at
        number of support vectors. 
        """
        try:
            import svm
        except ImportError as error:
            return

        svm = LibSVM()
        svm.setC(10.0)
        svm.setEpsilon(0.1)
        svm.setSvmType("Epsilon_SVR")

        numExamples = 100
        numFeatures = 10
        X = numpy.random.randn(numExamples, numFeatures)
        c = numpy.random.randn(numFeatures)

        y = numpy.dot(X, numpy.array([c]).T).ravel() + numpy.random.randn(100)

        print("This is the SVM regression")
        print("===================================================")
        svm.learnModel(X, y)
        
        svm.setEpsilon(0.5)
        svm.learnModel(X, y)

        svm.setEpsilon(0.01)
        svm.learnModel(X, y)

        #There should be fewer SVs as epsilon increases

    def testPredict(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 100
        numFeatures = 10
        X = numpy.random.randn(numExamples, numFeatures)
        c = numpy.random.randn(numFeatures)

        y = numpy.dot(X, numpy.array([c]).T).ravel()
        y = numpy.array(y > 0, numpy.int32)*2 -1

        svm = LibSVM()

        svm.learnModel(X, y)
        y2, d = svm.predict(X, True)

        self.assertTrue((numpy.sign(d) == y2).all())

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()

==> apgl-0.6.5.tar.gz: apgl-0.6.5/apgl/predictors/test/LibSVMTest.py <==
'''
Created on 24 Jul 2009

@author: charanpal
'''
import unittest
import logging
import numpy
import apgl
from apgl.predictors.LibSVM import LibSVM
from apgl.data import * 
from apgl.util import *


@apgl.skipIf(not apgl.checkImport('svm'), 'Module svm is required')
class LibSVMTest(unittest.TestCase):
    def setUp(self):
        try:
            import svm
        except ImportError as error:
            logging.debug(error)
            return 

        numExamples = 100
        numFeatures = 10
        eg = ExamplesGenerator()

        self.X, self.y = eg.generateBinaryExamples(numExamples, numFeatures)
        self.svm = LibSVM()

    def tearDown(self):
        pass

    def testLearnModel(self):
        try:
            import svm
        except ImportError as error:
            return

        self.svm.learnModel(self.X, self.y)
        predY, decisions = self.svm.classify(self.X)
        y = self.y

        e = Evaluator.binaryError(y, predY)
        logging.debug("Error rate : "  + str(e))

        #Test for wrong labels
        numExamples = 6
        X = numpy.array([[-3], [-2], [-1], [1], [2] ,[3]], numpy.float)
        y = numpy.array([[-1], [-1], [-1], [1], [1] ,[5]])


        self.assertRaises(ValueError, self.svm.learnModel, X, y)

    def testSetErrorCost(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 1000
        numFeatures = 100
        eg = ExamplesGenerator()
        X, y = eg.generateBinaryExamples(numExamples, numFeatures)
        svm = LibSVM()

        C = 0.1
        kernel = "linear"
        kernelParam = 0
        svm.setKernel(kernel, kernelParam)
        svm.setC(C)

        svm.setErrorCost(0.1)
        svm.learnModel(X, y)
        predY, decisions = svm.classify(X)
        e1 = Evaluator.binaryErrorP(y, predY)

        svm.setErrorCost(0.9)
        svm.learnModel(X, y)
        predY, decisions = svm.classify(X)
        e2 = Evaluator.binaryErrorP(y, predY)

        self.assertTrue(e1 > e2)

    def testClassify(self):
        try:
            import svm
        except ImportError as error:
            return

        self.svm.learnModel(self.X, self.y)
        predY, decisions = self.svm.classify(self.X)
        y = self.y

        e = Evaluator.binaryError(y, predY)

        #Now, permute examples
        perm = numpy.random.permutation(self.X.shape[0])
        predY, decisions = self.svm.classify(self.X[perm, :])
        y = y[perm]

        e2 = Evaluator.binaryError(y, predY)

        self.assertEquals(e, e2)

    def testEvaluateCv(self):
        try:
            import svm
        except ImportError as error:
            return

        folds = 10
        (means, vars) = self.svm.evaluateCv(self.X, self.y, folds)

        self.assertTrue((means <= 1).all())
        self.assertTrue((means>= 0).all())
        self.assertTrue((vars <= 1).all())
        self.assertTrue((vars>= 0).all())

    def testCvModelSelection(self):
        try:
            import svm
        except ImportError as error:
            return

        kernel = "linear"
        kernelParams = [0.1, 0.2, 0.4]
        Cs = [1.0, 0.5, 0.25, 0.125]
        errorCosts = [0.8, 0.9]
        folds = 5

        results = self.svm.cvModelSelection(self.X, self.y,Cs, kernelParams, kernel, folds, errorCosts)

        self.assertTrue(results["C"] in Cs)
        self.assertTrue(results["kp"] in kernelParams)
        self.assertTrue(results["errorCost"] in errorCosts)
        self.assertTrue(results["error"] >= 0 and results["error"] <= 1)
        

        #Test gaussian kernel
        kernel= "gaussian"
        results = self.svm.cvModelSelection(self.X, self.y ,Cs, kernelParams, kernel, folds, errorCosts)

        self.assertTrue(results["C"] in Cs)
        self.assertTrue(results["kp"] in kernelParams)
        self.assertTrue(results["errorCost"] in errorCosts)
        self.assertTrue(results["error"] >= 0 and results["error"] <= 1)


        #Test polynomial kernel
        kernel= "polynomial"
        kernelParams = [1, 2, 3]
        results = self.svm.cvModelSelection(self.X, self.y,Cs, kernelParams, kernel, folds, errorCosts)

        self.assertTrue(results["C"] in Cs)
        self.assertTrue(results["kp"] in kernelParams)
        self.assertTrue(results["errorCost"] in errorCosts)
        self.assertTrue(results["error"] >= 0 and results["error"] <= 1)

        #Try a different error function
        results = self.svm.cvModelSelection(self.X, self.y,Cs, kernelParams, kernel, folds, errorCosts, errorMethod=Evaluator.balancedError)

    def testGetModel(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 50
        numFeatures = 3
        eg = ExamplesGenerator()

        X, y = eg.generateBinaryExamples(numExamples, numFeatures)
        svm = LibSVM()
        svm.learnModel(X, y)

        weights, b  = svm.getWeights()

        logging.debug(weights)
        logging.debug(b)

    def testGetWeights(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 6
        X = numpy.array([[-3], [-2], [-1], [1], [2] ,[3]], numpy.float64)
        y = numpy.array([[-1], [-1], [-1], [1], [1] ,[1]])

        svm = LibSVM()
        svm.learnModel(X, y.ravel())

        weights, b  = svm.getWeights()

        self.assertTrue(weights[0] > 0)

        #Let's see if we can compute the decision values 
        y, decisions = svm.classify(X)

        decisions2 = numpy.zeros(numExamples)

        for i in range(numExamples):
            decisions2 = numpy.dot(X, weights) - b

        self.assertTrue((decisions == decisions2).all())
        predY = numpy.sign(decisions2)

        self.assertTrue((y.ravel() == predY).all())

    def testSetEpsilon(self):
        try:
            import svm
        except ImportError as error:
            return

        logging.debug("Learning model with epsilon as default")
        self.svm.learnModel(self.examplesList)

        logging.debug("Learning model with epsilon as 0.1")
        self.svm.setTermination(0.1)
        self.svm.learnModel(self.examplesList)
        logging.debug("Done")

    def testSetSvmType(self):
        try:
            import svm
        except ImportError as error:
            return

       
        numExamples = 100
        numFeatures = 10
        X = numpy.random.randn(numExamples, numFeatures)
        c = numpy.random.randn(numFeatures)

        y = numpy.dot(X, numpy.array([c]).T).ravel()
        y2 = numpy.array(y > 0, numpy.int32)*2 -1 
        
        svm = LibSVM()

        svm.setSvmType("Epsilon_SVR")
        svm.learnModel(self.X, self.y)
        yp = svm.predict(X)
        logging.debug("Error on real labels ")
        logging.debug((Evaluator.rootMeanSqError(y, yp)))

        #The values of y look correct so it looks like its working 
        #logging.debug(y[0:5])
        #logging.debug(yp[0:5])

        svm.setSvmType("C_SVC")
        svm.learnModel(X, y2)
        yp2 = svm.predict(X)

        logging.debug("Error on predicted labels ")
        logging.debug((Evaluator.binaryError(y2, yp2)))
        logging.debug((y2[0:5]))
        logging.debug((yp2[0:5]))

    def testCvModelSelection2(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 100
        numFeatures = 10
        X = numpy.random.randn(numExamples, numFeatures)
        c = numpy.random.randn(numFeatures)

        y = numpy.dot(X, numpy.array([c]).T).ravel()

        examplesList = ExamplesList(numExamples)
        examplesList.addDataField("X", X)
        examplesList.addDataField("y", y)
        examplesList.setDefaultExamplesName("X")
        examplesList.setLabelsName("y")

        kernel = "linear"
        kernelParams = [0.1, 0.2, 0.4]
        Cs = [1.0, 0.5, 0.25, 0.125]
        errorCosts = [1.0]
        epsilons = [0.05, 0.1, 0.2, 0.5]
        folds = 5

        svm = LibSVM()
        svm.setSvmType("Epsilon_SVR")
        best = svm.cvModelSelection(self.X, self.y,Cs, kernelParams, kernel, folds, errorCosts, epsilons=epsilons, errorMethod=Evaluator.rootMeanSqError)

        logging.debug(best)

    def testSaveParams(self):
        try:
            import svm
        except ImportError as error:
            return

        svm = LibSVM()
        svm.setC(10.5)
        svm.setEpsilon(12.1)
        svm.setErrorCost(1.8)
        svm.setSvmType("Epsilon_SVR")
        svm.setTermination(0.12)
        svm.setKernel("gaussian", 0.43)

        outputDir = PathDefaults.getOutputDir()
        fileName = outputDir + "test/testSvmParams"
        svm.saveParams(fileName)

        svm2 = LibSVM()
        svm2.loadParams(fileName)

        self.assertEquals(svm.getC(), 10.5)
        self.assertEquals(svm.getEpsilon(), 12.1)
        self.assertEqual(svm.getErrorCost(), 1.8)
        self.assertEqual(svm.getSvmType(), "Epsilon_SVR")
        self.assertEqual(svm.getTermination(), 0.12)
        self.assertEqual(svm.getKernel(), "gaussian")
        self.assertEqual(svm.getKernelParams(), 0.43)

    def testStr(self):
        try:
            import svm
        except ImportError as error:
            return

        svm = LibSVM()

        logging.debug(svm)

    def testSetEpsilon(self):
        """
        Test out the parameter for the regressive SVM, vary epsilon and look at
        number of support vectors. 
        """
        try:
            import svm
        except ImportError as error:
            return

        svm = LibSVM()
        svm.setC(10.0)
        svm.setEpsilon(0.1)
        svm.setSvmType("Epsilon_SVR")

        numExamples = 100
        numFeatures = 10
        X = numpy.random.randn(numExamples, numFeatures)
        c = numpy.random.randn(numFeatures)

        y = numpy.dot(X, numpy.array([c]).T).ravel() + numpy.random.randn(100)

        logging.debug("This is the SVM regression")
        logging.debug("===================================================")
        svm.learnModel(X, y)
        
        svm.setEpsilon(0.5)
        svm.learnModel(X, y)

        svm.setEpsilon(0.01)
        svm.learnModel(X, y)

        #There should be fewer SVs as epsilon increases

    def testPredict(self):
        try:
            import svm
        except ImportError as error:
            return

        numExamples = 100
        numFeatures = 10
        X = numpy.random.randn(numExamples, numFeatures)
        c = numpy.random.randn(numFeatures)

        y = numpy.dot(X, numpy.array([c]).T).ravel()
        y = numpy.array(y > 0, numpy.int32)*2 -1

        svm = LibSVM()

        svm.learnModel(X, y)
        y2, d = svm.predict(X, True)

        self.assertTrue((numpy.sign(d) == y2).all())

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()
